@@ -116,6 +116,7 @@ enum
 #define	CC_set_in_ansi_app(x)	(x->unicode |= CONN_ANSI_APP)
 #define	CC_is_in_unicode_driver(x)	(0 != (x->unicode & CONN_UNICODE_DRIVER))
 #define	CC_is_in_ansi_app(x)	(0 != (x->unicode & CONN_ANSI_APP))
+#define	CC_is_in_global_trans(x)	(NULL != (x)->asdum)
 #define	ALLOW_WCHAR(x)	(0 != (x->unicode & CONN_UNICODE_DRIVER) && 0 == (x->unicode & CONN_DISALLOW_WCHAR))
 
 #define CC_MALLOC_return_with_error(t, tp, s, x, m, ret) \
@@ -285,8 +286,7 @@ typedef struct
 	char		server[MEDIUM_REGISTRY_LEN];
 	char		database[MEDIUM_REGISTRY_LEN];
 	char		username[MEDIUM_REGISTRY_LEN];
-	char		password[MEDIUM_REGISTRY_LEN];
-	char		conn_settings[LARGE_REGISTRY_LEN];
+	pgNAME		password;
 	char		protocol[SMALL_REGISTRY_LEN];
 	char		port[SMALL_REGISTRY_LEN];
 	char		sslmode[16];
@@ -298,6 +298,7 @@ typedef struct
 	char		translation_dll[MEDIUM_REGISTRY_LEN];
 	char		translation_option[SMALL_REGISTRY_LEN];
 	char		focus_password;
+	pgNAME		conn_settings;
 	signed char	disallow_premature;
 	signed char	allow_keyset;
 	signed char	updatable_cursors;
@@ -488,6 +489,7 @@ struct ConnectionClass_
 	pthread_mutex_t		slock;
 #endif /* WIN_MULTITHREAD_SUPPORT */
 #ifdef	_HANDLE_ENLIST_IN_DTC_
+	UInt4		gTranInfo;
 	void		*asdum;
 #endif /* _HANDLE_ENLIST_IN_DTC_ */
 };
@@ -509,13 +511,41 @@ struct ConnectionClass_
 #define CONN_DONT_OVERWRITE		0
 #define CONN_OVERWRITE			1
 
+#ifdef	_HANDLE_ENLIST_IN_DTC_
+enum {
+	DTC_IN_PROGRESS	= 1L
+	,DTC_ENLISTED	= (1L << 1)
+	,DTC_REQUEST_EXECUTING	= (1L << 2)
+	,DTC_ISOLATED	= (1L << 3)
+	,DTC_PREPARE_REQUESTED	= (1L << 4)
+};
+#define	CC_set_dtc_clear(x)	((x)->gTranInfo = 0)
+#define	CC_set_dtc_enlisted(x)	((x)->gTranInfo |= (DTC_IN_PROGRESS | DTC_ENLISTED))
+#define	CC_no_dtc_enlisted(x)	((x)->gTranInfo &= (~DTC_ENLISTED))
+#define	CC_is_dtc_enlisted(x)	(0 != ((x)->gTranInfo & DTC_ENLISTED))
+#define	CC_set_dtc_executing(x)	((x)->gTranInfo |= DTC_REQUEST_EXECUTING)
+#define	CC_no_dtc_executing(x)	((x)->gTranInfo &= (~DTC_REQUEST_EXECUTING))
+#define	CC_is_dtc_executing(x)	(0 != ((x)->gTranInfo & DTC_REQUEST_EXECUTING))
+#define	CC_set_dtc_prepareRequested(x)	((x)->gTranInfo |= (DTC_PREPARE_REQUESTED))
+#define	CC_no_dtc_prepareRequested(x)	((x)->gTranInfo &= (~DTC_PREPARE_REQUESTED))
+#define	CC_is_dtc_prepareRequested(x)	(0 != ((x)->gTranInfo & DTC_PREPARE_REQUESTED))
+#define	CC_is_dtc_executing(x)	(0 != ((x)->gTranInfo & DTC_REQUEST_EXECUTING))
+#define	CC_set_dtc_isolated(x)	((x)->gTranInfo |= DTC_ISOLATED)
+#define	CC_is_idle_in_global_transaction(x)	(0 != ((x)->gTranInfo & DTC_PREPARE_REQUESTED) || (x)->gTranInfo == DTC_IN_PROGRESS)
+#endif /* _HANDLE_ENLIST_IN_DTC_ */
+
 
 /*	prototypes */
 ConnectionClass *CC_Constructor(void);
-void		CC_conninfo_init(ConnInfo *conninfo);
+enum { /* CC_conninfo_init option */
+	CLEANUP_FOR_REUSE	= 1L		/* reuse the info */
+	,COPY_GLOBALS		= (1L << 1) /* copy globals to drivers */
+};
+void		CC_conninfo_init(ConnInfo *conninfo, UInt4 option);
+void		CC_copy_conninfo(ConnInfo *to, const ConnInfo *from);
 char		CC_Destructor(ConnectionClass *self);
 int		CC_cursor_count(ConnectionClass *self);
-char		CC_cleanup(ConnectionClass *self);
+char		CC_cleanup(ConnectionClass *self, BOOL keepCommunication);
 char		CC_begin(ConnectionClass *self);
 char		CC_commit(ConnectionClass *self);
 char		CC_abort(ConnectionClass *self);
@@ -567,6 +597,8 @@ BOOL		SendSyncRequest(ConnectionClass *self);
 const		char *CurrCat(const ConnectionClass *self);
 const		char *CurrCatString(const ConnectionClass *self);
 
+void	CC_examine_global_transaction(ConnectionClass *self);
+
 /* CC_send_query options */
 enum {
 	IGNORE_ABORT_ON_CONN	= 1L /* not set the error result even when  */
