@@ -111,12 +111,10 @@ struct addrinfo
 #endif /* WIN32 */
 typedef void (WSAAPI *freeaddrinfo_func) (struct addrinfo *); 
 typedef int (WSAAPI *getaddrinfo_func) (const char *, const char *,
-#ifndef	__CYGWIN__
-	const
-#endif
-	struct addrinfo *, struct addrinfo **); 
+	const struct addrinfo *, struct addrinfo **); 
 typedef int (WSAAPI *getnameinfo_func) (const struct sockaddr *,
 	socklen_t, char *, size_t, char *, size_t, int);
+typedef int (WSAAPI *inet_pton_func) (int, const char *, void *);
 
 #ifdef	MSG_NOSIGNAL
 #define	SEND_FLAG MSG_NOSIGNAL
@@ -236,7 +234,5 @@ Int4		SOCK_flush_output(SocketClass *self);
 UCHAR		SOCK_get_next_byte(SocketClass *self, BOOL peek);
 void		SOCK_put_next_byte(SocketClass *self, UCHAR next_byte);
 Int4		SOCK_get_response_length(SocketClass *self);
-void		SOCK_clear_error(SocketClass *self);
-UInt4		SOCK_skip_n_bytes(SocketClass *self, UInt4 skip_length);
 
 #endif /* __SOCKET_H__ */
