@@ -18,8 +18,9 @@ typedef enum {
 	,NegotiateService = (1L << 2)
 } SSPI_Service;
 
+void	LeaveSSPIService();
 void	ReleaseSvcSpecData(SocketClass *self, UInt4);
-int	StartupSspiService(SocketClass *self, SSPI_Service svc, const void *opt);
+int	StartupSspiService(SocketClass *self, SSPI_Service svc, const void *opt, int *bReconnect);
 int	ContinueSspiService(SocketClass *self, SSPI_Service svc, const void *opt);
 int	SSPI_recv(SocketClass *self, void *buf, int len);
 int	SSPI_send(SocketClass *self, const void *buf, int len);
