@@ -29,3 +29,5 @@ extern void test_connect(void);
 extern void test_disconnect(void);
 extern void print_result_meta(HSTMT hstmt);
 extern void print_result(HSTMT hstmt);
+extern const char *datatype_str(SQLSMALLINT datatype);
+extern const char *nullable_str(SQLSMALLINT nullable);
